@@ -36,6 +36,7 @@ from scipy.interpolate import splev, splprep
 import matplotlib.pyplot as plt
 import platform
 import pymeshfix
+from .sv_interface import ROM
 #from wrapt_timeout_decorator import *
 
 class tree:
@@ -629,6 +630,14 @@ class tree:
         #for id,branch in enumerate(branches):
         #    branch_ids = []
         #    for seg in branch
+
+        if outdir is None:
+            outdir = os.getcwd()
+        outdir = outdir + os.sep + folder
+
+        if not os.path.exists(outdir):
+            os.mkdir(outdir)
+
         def make_points(x,y,z):
             """Helper to make XYZ points"""
             return np.column_stack((x, y, z))
@@ -650,31 +659,47 @@ class tree:
             spline_data_normal = splev(n,spline[0],der=1)
             points = make_points(spline_data[0],spline_data[1],spline_data[2])
             normal = make_points(spline_data_normal[0],spline_data_normal[1],spline_data_normal[2])
-            normal = normal/np.linalg.norm(normal,axis=1)
+            normal = normal/np.linalg.norm(normal,axis=1).reshape(-1,1)
             poly_line = lines_from_points(points)
             poly_line['VesselId'] = np.ones(num_points,dtype=int)*ind
             poly_line['MaximumInscribedSphereRadius'] = spline_data[3]
             poly_line['CenterlineSectionArea'] = np.pi*spline_data[3]**2
-            poly_line.point_data.set_vector(normal,'CenterlineSectionNormal')
+            poly_line.point_data.set_array(normal,'CenterlineSectionNormal')
             polys.append(poly_line)
 
         for ind in range(len(polys)):
-            cent_ids = np.zeros((polys[ind],len(polys)),dtype=int)
+            cent_ids = np.zeros((polys[ind].n_points,len(polys)),dtype=int)
             polys[ind].point_data.set_array(cent_ids,'CenterlineId')
             polys[ind].point_data['CenterlineId'][:,ind] = 1
 
-        bifucation_point_ids = [] # polys ind index, polys jnd index, polys jnd point index
+        bifurcation_point_ids = [] # polys ind index, polys jnd index, polys jnd point index
         for ind in range(1,len(polys)):
-            for jnd in range(len(polys)):
-                if jnd == ind:
-                    continue
-                closest_point_id = polys[jnd].find_closest_point(polys[ind].points[0])
+            current = ind
+            count = 0
+            while current != 0:
+                #if jnd == ind:
+                #    continue
+                jnd = None
+                closest_dist = np.inf
+                for i in range(len(polys)):
+                    if i == current:
+                        continue
+                    closest_pt_id = polys[i].find_closest_point(polys[current].points[0])
+                    closest_point = polys[i].points[closest_pt_id]
+                    closest_dist_tmp = np.linalg.norm(polys[current].points[0] - closest_point)
+                    if closest_dist_tmp < closest_dist:
+                        jnd = i
+                        closest_dist = closest_dist_tmp
+                closest_point_id = polys[jnd].find_closest_point(polys[current].points[0])
                 closest_point    = polys[jnd].points[closest_point_id]
-                dist             = np.linalg.norm(polys[ind].points[0] - closest_point)
-                if dist < polys[ind].point_data['Radius'][0]:
-                    polys[jnd].point_data['CenterlineId'][0:closest_point_id+1,ind] = 1
-                    bifurcation_point_ids.append([ind,jnd,closest_point_id])
-
+                dist             = np.linalg.norm(polys[current].points[0] - closest_point)
+                if count == 0:
+                    bifurcation_point_ids.append([ind,jnd,closest_point_id,closest_point])
+                #if dist < polys[current].point_data['MaximumInscribedSphereRadius'][0]:
+                polys[jnd].point_data['CenterlineId'][0:closest_point_id+1,ind] = 1
+                #bifurcation_point_ids.append([ind,jnd,closest_point_id,closest_point])
+                current = jnd
+                count += 1
         # Determine Branch Temp Ids
         branch_tmp_count = 0
         for ind in range(len(polys)):
@@ -685,7 +710,7 @@ class tree:
             tmp_split.sort()
             tmp_split.insert(0,0)
             tmp_split.append(None)
-            branch_tmp_ids = np.zeros(len(polys.points.shape[0]),dtype=int)
+            branch_tmp_ids = np.zeros(polys[ind].points.shape[0],dtype=int)
             for i in range(1,len(tmp_split)):
                 branch_tmp_ids[tmp_split[i-1]:tmp_split[i]] = branch_tmp_count
                 branch_tmp_count += 1
@@ -695,6 +720,13 @@ class tree:
             tmp_bif = []
             tmp_bif_pt = []
             for bif in bifurcation_point_ids:
+                if ind > 0:
+                    rad = polys[ind].point_data['MaximumInscribedSphereRadius'][0]
+                    pt  = polys[ind].points[0,:]
+                    b   = np.argwhere(np.linalg.norm(polys[ind].points - pt,axis=1)<rad).flatten().tolist()
+                    b_pt = b.pop(b.index(0))
+                    tmp_bif.extend(b)
+                    tmp_bif_pt.append(0)
                 if bif[1] == ind:
                     rad = polys[ind].point_data['MaximumInscribedSphereRadius'][bif[2]]
                     pt  = polys[ind].points[bif[2]]
@@ -702,16 +734,105 @@ class tree:
                     b_pt = b.pop(b.index(bif[2]))
                     tmp_bif.extend(b)
                     tmp_bif_pt.append(b_pt)
-            tmp_bif = np.array(tmp_bif)
-            tmp_bif_pt = np.array(tmp_bif_pt)
-            bifurcation_id_tmp = np.ones(len(polys.points.shape[0]),dtype=int)*-1
+            tmp_bif = np.array(tmp_bif,dtype=int)
+            tmp_bif_pt = np.array(tmp_bif_pt,dtype=int)
+            bifurcation_id_tmp = np.ones(polys[ind].points.shape[0],dtype=int)*-1
             bifurcation_id_tmp[tmp_bif] = 2
             bifurcation_id_tmp[tmp_bif_pt] = 1
             polys[ind].point_data['BifurcationIdTmp'] = bifurcation_id_tmp
 
-        # Assign branch ids
-        # np.argwhere(polys[ind].point_data['BifurcationIdTmp']==)
-        return polys
+        bif_id_count = 0
+        for ind in range(len(polys)):
+            bif_id = []
+            new = True
+            for jnd in range(polys[ind].n_points):
+                if polys[ind].point_data['BifurcationIdTmp'][jnd] > 0 and new:
+                    bif_id.append(bif_id_count)
+                    #bif_id_count_next = 1 + bif_id_count
+                    new = False
+                elif polys[ind].point_data['BifurcationIdTmp'][jnd] > 0 and not new:
+                    bif_id.append(bif_id_count)
+                    #bif_id_count += 1
+                    #new = True
+                elif polys[ind].point_data['BifurcationIdTmp'][jnd] < 0 and not new:
+                    bif_id.append(-1)
+                    bif_id_count += 1
+                    new = True
+                else:
+                    bif_id.append(-1)
+            bif_id = np.array(bif_id)
+            polys[ind].point_data['BifurcationId'] = bif_id
+
+        branch_id_count = 0
+        for ind in range(len(polys)):
+            branch_id = []
+            for jnd in range(polys[ind].n_points):
+                new = False
+                if polys[ind].point_data['BifurcationId'][jnd] < 0:
+                    branch_id.append(branch_id_count)
+                    new = False
+                elif polys[ind].point_data['BifurcationId'][jnd] > 0 and not new:
+                    branch_id.append(-1)
+                    branch_id_count += 1
+                    new = True
+                else:
+                    branch_id.append(-1)
+            branch_id = np.array(branch_id)
+            polys[ind].point_data['BranchId'] = branch_id
+
+        # Set Path Values for Branches and Bifurcations also obtain outlet branches
+        outlets = []
+        for ind in range(len(polys)):
+            branch_ids = list(set(polys[ind].point_data['BranchId'].tolist()))
+            outlets.append(max(branch_ids))
+            branch_ids.sort()
+            path_init  = np.zeros(polys[ind].points.shape[0])
+            polys[ind].point_data['Path'] = path_init
+            if branch_ids[0] == -1:
+                branch_ids = branch_ids[1:]
+            for b_idx in branch_ids:
+                poly_pt_ids = np.argwhere(polys[ind].point_data['BranchId']==b_idx).flatten()
+                poly_pt_path = np.cumsum(np.insert(np.linalg.norm(np.diff(polys[ind].points[poly_pt_ids],axis=0),axis=1),0,0))
+                polys[ind].point_data['Path'][poly_pt_ids] = poly_pt_path
+            bif_ids = list(set(polys[ind].point_data['BifurcationId'].tolist()))
+            bif_ids.sort()
+            if bif_ids[0] == -1:
+                bif_ids = bif_ids[1:]
+            for bif_idx in bif_ids:
+                poly_pt_ids = np.argwhere(polys[ind].point_data['BifurcationId']==bif_idx).flatten()
+                poly_pt_path = np.cumsum(np.insert(np.linalg.norm(np.diff(polys[ind].points[poly_pt_ids],axis=0),axis=1),0,0))
+                polys[ind].point_data['Path'][poly_pt_ids] = poly_pt_path
+
+        # Set Point Ids
+        Global_node_count = 0
+        branch_starts = []
+        for ind in range(len(polys)):
+            GlobalNodeId = list(range(Global_node_count,Global_node_count+polys[ind].n_points))
+            Global_node_count = GlobalNodeId[-1]+1
+            if ind < len(polys):
+                branch_starts.append(Global_node_count)
+            GlobalNodeId = np.array(GlobalNodeId)
+            polys[ind].point_data['GlobalNodeId'] = GlobalNodeId
+
+        # Merge and Connect Lines
+        centerlines_all = polys[0]
+        for ind in range(1,len(polys)):
+            closest_pt_id = centerlines_all.find_closest_point(polys[ind].points[0])
+            closest_next_id = centerlines_all.points.shape[0]
+            centerlines_all = centerlines_all.merge(polys[ind])
+            new_line = [2,closest_pt_id,closest_next_id]
+            centerlines_all.lines = np.hstack((centerlines_all.lines,np.array(new_line)))
+        # Connect lines among spline branches
+        #for branch_global_node_id in branch_starts:
+        #    pt_idx = np.argwhere(centerline_all.point_data['GlobalNodeId']==branch_global_node_id)
+        #    remaining_points =
+
+        # Generate 1D Solver Files
+        centerlines_all.save(outdir+os.sep+'centerlines.vtp')
+        #param = ROM.parameters.Parameters()
+        #param.output_directory = outdir
+        #param
+        return centerlines_all,polys
 
     def collision_free(self,outside_vessels,radius_buffer=0.01):
         return no_outside_collision(self,outside_vessels,radius_buffer=radius_buffer)
@@ -915,7 +1036,7 @@ class forest:
             for i in tqdm(range(number_of_branches),desc="Adding branches"):
                 compete_add(self,network_ids=network_id,radius_buffer=radius_buffer)
 
-    def show(self,show=True,resolution=100,final=False,merged_trees=False):
+    def show(self,show=True,resolution=100,final=False,merged_trees=False,background='white',off_screen=False):
         if merged_trees:
             colors = ['r','b','g','y']
             plotter = pv.Plotter()
@@ -982,9 +1103,9 @@ class forest:
                                                   resolution=resolution))
         """
         if show:
-            plot = pv.Plotter()
+            plot = pv.Plotter(off_screen=off_screen)
             colors = ['r','b','g','y']
-            #plot.set_background(color=[253,250,219])
+            plot.set_background(background)
             for model_idx,model_network in enumerate(model_networks):
                 for color_idx, model_tree in enumerate(model_network):
                     for model in model_tree:
